@@ -270,6 +270,181 @@ class EPD2in7(WaveshareFull):
         self.send_data(0xa5)
 
 
+class EPD3in7(WaveshareFull):
+    """Waveshare 3.7" - monochrome"""
+
+    def __init__(self):
+        super().__init__(name='3.7" BW', width=280, height=480)
+
+    # The 3.7" display hardware supports B/W and 4-shade grayscale modes, but the
+    # grayscale mode is not yet implemented here. Reference code in Waveshare's repo:
+    # https://github.com/waveshare/e-Paper/blob/master/RaspberryPi_JetsonNano/python/lib/waveshare_epd/epd3in7.py
+    # Updating the screen with partial refresh takes approximately 600-900 ms. It is
+    # also not yet implemented here. Reference code in Waveshare's repo:
+    # https://github.com/waveshare/e-Paper/blob/master/RaspberryPi_JetsonNano/c/lib/e-Paper/EPD_3in7.c#L549
+
+    lut_1Gray_GC  = [
+        0x2A,0x05,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
+        0x05,0x2A,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
+        0x2A,0x15,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
+        0x05,0x0A,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
+        0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
+        0x00,0x02,0x03,0x0A,0x00,0x02,0x06,0x0A,0x05,0x00,
+        0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
+        0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
+        0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
+        0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
+        0x22,0x22,0x22,0x22,0x22
+    ]
+
+    lut_1Gray_DU  = [
+        0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
+        0x01,0x2A,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
+        0x0A,0x55,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
+        0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
+        0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
+        0x00,0x00,0x05,0x05,0x00,0x05,0x03,0x05,0x05,0x00,
+        0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
+        0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
+        0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
+        0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
+        0x22,0x22,0x22,0x22,0x22
+    ]
+
+    lut_1Gray_A2  = [
+        0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
+        0x0A,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
+        0x05,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
+        0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00, 
+        0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00, 
+        0x00,0x00,0x03,0x05,0x00,0x00,0x00,0x00,0x00,0x00, 
+        0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00, 
+        0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00, 
+        0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00, 
+        0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00, 
+        0x22,0x22,0x22,0x22,0x22
+    ]
+
+    def wait_until_idle(self):
+        while self.digital_read(self.BUSY_PIN) == 1:      #  0: idle, 1: busy
+            self.delay_ms(10)
+
+    def init(self, **kwargs):
+        if self.epd_init() != 0:
+            return -1
+        # EPD hardware init start
+        self.reset()
+
+
+        self.send_command(0x12)
+        self.delay_ms(300)
+        
+        self.send_command(0x46)
+        self.send_data(0xF7)
+        self.wait_until_idle()
+        self.send_command(0x47)
+        self.send_data(0xF7)
+        self.wait_until_idle()
+        
+        self.send_command(0x01) # setting gaet number
+        self.send_data(0xDF)
+        self.send_data(0x01)
+        self.send_data(0x00)
+
+        self.send_command(0x03) # set gate voltage
+        self.send_data(0x00)
+
+        self.send_command(0x04) # set source voltage
+        self.send_data(0x41)
+        self.send_data(0xA8)
+        self.send_data(0x32)
+
+        self.send_command(0x11) # set data entry sequence
+        self.send_data(0x03)
+
+        self.send_command(0x3C) # set border 
+        self.send_data(0x00)
+        
+        self.send_command(0x0C) # set booster strength
+        self.send_data(0xAE)
+        self.send_data(0xC7)
+        self.send_data(0xC3)
+        self.send_data(0xC0)
+        self.send_data(0xC0)
+
+        self.send_command(0x18) # set internal sensor on
+        self.send_data(0x80)
+         
+        self.send_command(0x2C) # set vcom value
+        self.send_data(0x44)
+        
+        self.send_command(0x37) # set display option, these setting turn on previous function
+        self.send_data(0x00)     #can switch 1 gray or 4 gray
+        self.send_data(0xFF)
+        self.send_data(0xFF)
+        self.send_data(0xFF)
+        self.send_data(0xFF)  
+        self.send_data(0x4F)
+        self.send_data(0xFF)
+        self.send_data(0xFF)
+        self.send_data(0xFF)
+        self.send_data(0xFF)
+
+        self.send_command(0x44) # setting X direction start/end position of RAM
+        self.send_data(0x00)
+        self.send_data(0x00)
+        self.send_data(0x17)
+        self.send_data(0x01)
+
+        self.send_command(0x45) # setting Y direction start/end position of RAM
+        self.send_data(0x00)
+        self.send_data(0x00)
+        self.send_data(0xDF)
+        self.send_data(0x01)
+
+        self.send_command(0x22) # Display Update Control 2
+        self.send_data(0xCF)
+        self.load_lut(self.lut_1Gray_A2)
+        return 0
+
+    def load_lut(self, lut):
+        self.send_command(0x32)
+        for i in range(0, 105):
+            self.send_data(lut[i])
+
+    def display_frame(self, frame_buffer, *args):
+        if frame_buffer:
+            self.send_command(0x4E)
+            self.send_data(0x00)
+            self.send_data(0x00)
+            self.send_command(0x4F)
+            self.send_data(0x00)
+            self.send_data(0x00)
+
+            self.send_command(0x24)
+            for j in range(0, self.height):
+                for i in range(0, int(self.width / 8)):
+                    self.send_data(frame_buffer[i + j * int(self.width / 8)])   
+
+            self.send_command(0x20)
+            self.wait_until_idle()   
+
+    def sleep(self):
+        self.send_command(0X50) # DEEP_SLEEP_MODE
+        self.send_data(0xf7)
+        self.send_command(0X02) #power off
+        self.send_command(0X07) #deep sleep
+        self.send_data(0xA5)
+
+    def reset(self):
+        self.digital_write(self.RST_PIN, 1)
+        self.delay_ms(200)
+        self.digital_write(self.RST_PIN, 0)
+        self.delay_ms(5)
+        self.digital_write(self.RST_PIN, 1)
+        self.delay_ms(200)
+
+
 class EPD7in5(WaveshareFull):
     """Waveshare 7.5" - monochrome"""
 
